from django.core.validators import FileExtensionValidator
from django.db import models
from django.utils.translation import pgettext_lazy

from ..const import (
    INDEX_CHOICES,
    FOLLOW_CHOICES,
    DEFAULT_OBJECT_TYPES,
    DEFAULT_TWITTER_TYPES
)
from ..settings import (
    SEO_OG_TYPES,
    SEO_TWITTER_TYPES, 
    SEO_IMAGE_EXTENSIONS,
    SEO_IMAGE_WIDTH,
    SEO_IMAGE_HEIGHT
)
from ..utils import image_upload_to

__all__ = (
    'BaseSeoModel',
)


class BaseSeoModel(models.Model):
    """
    Base Seo model

    Attrs:
        object_type (CharField): object type
        twitter_type (CharField): twitter type
        index (CharField): robots index value
        follow (CharField): robots follow value
        title (CharField): meta title
        keywords (CharField): meta keywords
        description (CharField): meta description
        image (VersatileImageField): OG image
        alt (CharField): alt text for image
        h1 (CharField): H1 title for page
        top_text (TextField): text field for top part of the page
        bottom_text (TextField): text field for bottom part of the page
    """
    object_type = models.CharField(
        pgettext_lazy('Base seo model', 'Open graph type'),
        max_length=40,
        blank=True,
        choices=SEO_OG_TYPES,
        default=DEFAULT_OBJECT_TYPES[0][0]
    )
    twitter_type = models.CharField(
        pgettext_lazy('Base seo model', 'Twitter type'),
        max_length=40,
        blank=True,
        choices=SEO_TWITTER_TYPES,
        default=DEFAULT_TWITTER_TYPES[0][0]
    )
    index = models.CharField(
        pgettext_lazy('Base seo model', 'Index robots value'),
        max_length=15,
        blank=True,
        choices=INDEX_CHOICES,
        default=INDEX_CHOICES[0][0]
    )
    follow = models.CharField(
        pgettext_lazy('Base seo model', 'Follow robots value'),
        max_length=15,
        blank=True,
        choices=FOLLOW_CHOICES,
        default=FOLLOW_CHOICES[0][0]
    )
    canonical = models.CharField(
        pgettext_lazy('Base seo model', 'Canonical'),
        max_length=255,
        blank=True
    )
    title = models.CharField(
        pgettext_lazy('Base seo model', 'Seo Title'),
        max_length=255,
        blank=True
    )
    keywords = models.TextField(
        pgettext_lazy('Base seo model', 'Meta Keywords'),
        blank=True,
    )
    description = models.TextField(
        pgettext_lazy('Base seo model', 'Meta Description'),
        blank=True,
    )
    image = models.ImageField(
        pgettext_lazy('Base seo model', 'Image'),
        upload_to=image_upload_to,
        blank=True, null=True,
        validators=[FileExtensionValidator(SEO_IMAGE_EXTENSIONS)]
    )
    width = models.PositiveIntegerField(
        pgettext_lazy('Base seo model', 'Image width'),
        default=SEO_IMAGE_WIDTH
    )
    height = models.PositiveIntegerField(
        pgettext_lazy('Base seo model', 'Image height'),
        default=SEO_IMAGE_HEIGHT
    )
    alt = models.CharField(
        pgettext_lazy('Base seo model', 'Image alt text'),
        max_length=255,
        blank=True
    )
    h1 = models.CharField(
        pgettext_lazy('Base seo model', 'H1 title'),
        max_length=255,
        blank=True
    )
    seo_text = models.TextField(
        pgettext_lazy('Base seo model', 'Seo text'),
        blank=True,
        help_text=pgettext_lazy(
            'Base seo model',
            'Can be useful for some static pages or some objects (like product category).'
        ),
    )

    class Meta:
        abstract = True

    def delete(self, *args, **kwargs):
        super().delete(*args, **kwargs)
        self.image.delete(save=False)



from typing import Dict

from ..models.instance_based import ModelInstanceSeo
from ..models.view_based import ViewSeo

__all__ = (
    'ViewSeoMixin',
    'ModelInstanceViewSeoMixin'
)


class ViewSeoMixin:
    """
    Mixin to add meta tags from ViewSeo found by given template
    """
    seo_view = ''  # type: str

    def get_seo(self) -> Dict[str, ViewSeo]:
        return {
            'seo': (
                ViewSeo
                .objects
                .filter(view=self.seo_view)
                .first()
            )
        }

    def get_context_data(self, **kwargs) -> Dict:
        kwargs.update(self.get_seo())
        return super().get_context_data(**kwargs)


class ModelInstanceViewSeoMixin(ViewSeoMixin):
    """
    Mixin to fetch seo data for current object
    """
    def get_seo(self):
        return {
            'seo': (
                ModelInstanceSeo
                .objects
                .filter_by_instance(self.object)
                .first()
            )
        }



from modeltranslation.translator import translator, TranslationOptions

from .models.instance_based import ModelInstanceSeo
from .models.view_based import ViewSeo

__all__ = (
    'ModelInstanceSeoTranslationOptions',
    'ViewSeoTranslationOptions'
)


class ModelInstanceSeoTranslationOptions(TranslationOptions):
    fields = (
        'title', 
        'keywords', 
        'description',
        'alt', 
        'h1', 
        'seo_text'
    )


class ViewSeoTranslationOptions(ModelInstanceSeoTranslationOptions):
    pass


translator.register(ModelInstanceSeo, ModelInstanceSeoTranslationOptions)
translator.register(ViewSeo, ViewSeoTranslationOptions)



from django.test import TestCase
from django.test.client import RequestFactory
from django.template import Context, Template

from ..models.view_based import ViewSeo

__all__ = (
    'GetSeoDataTemplateTagTest',
)


class GetSeoDataTemplateTagTest(TestCase):

    def setUp(self):
        self.factory = RequestFactory()

    def test_rendered(self):
        seo = ViewSeo(
            title='1',
            description='1',
            keywords='1,2,3',
            object_type='article'
        )
        request = self.factory.get('/')
        context = Context({'seo': seo}, {'request': request})
        context.request = request
        template_to_render = Template(
            '{% load seo %}'
            '{% get_seo_data %}'
        )
        rendered_template = template_to_render.render(context)
        self.assertInHTML(
            f'<title>{seo.get_meta_title()}</title>',
            rendered_template
        )
        self.assertInHTML(
            f'<meta name="description" content="{seo.get_meta_description()}">',
            rendered_template
        )
        self.assertInHTML(
            f'<meta name="keywords" content="{seo.get_meta_keywords()}">',
            rendered_template
        )
        self.assertInHTML(
            f'<meta property="og:type" content="{seo.get_opengraph_type()}">',
            rendered_template
        )
        self.assertInHTML(
            f'<meta property="og:title" content="{seo.get_meta_title()}">',
            rendered_template
        )
        self.assertInHTML(
            f'<meta property="og:description" content="{seo.get_meta_description()}">',
            rendered_template
        )
        self.assertInHTML(
            f'<meta property="og:image" content="{request.build_absolute_uri(seo.get_meta_image())}">',
            rendered_template
        )
        self.assertInHTML(
            f'<meta name="twitter:card" content="{seo.get_twitter_type()}">',
            rendered_template
        )



from django.db import models
from django.utils.translation import pgettext_lazy

from .base import BaseSeoModel
from ..mixins.models import SeoTagsMixin
from .. import settings

SEO_VIEWS_CHOICES = settings.SEO_VIEWS_CHOICES

__all__ = (
    'ViewSeo',
)


class ViewSeo(SeoTagsMixin, BaseSeoModel):
    """
    Seo model to add seo staff for views

    Attrs:
        view (CharField): custom view name
    """

    view = models.CharField(
        pgettext_lazy("View seo", "View"),
        max_length=100,
        choices=SEO_VIEWS_CHOICES,
        unique=True
    )

    class Meta:
        verbose_name = pgettext_lazy("Model instance seo", "View seo")
        verbose_name_plural = pgettext_lazy("Model instance seo", "View seo")

    def __str__(self) -> str:
        return self.title



from importlib import import_module

from django import forms
from django.apps import apps
from django.contrib import admin
from django.contrib.contenttypes.admin import GenericStackedInline

from ..settings import SEO_HTML_ADMIN_WIDGET

__all__ = (
    'get_admin_base_class',
    'get_admin_inline_base_class',
    'get_html_field_widget'
)


def get_admin_base_class():
    """
    Return base admin class
    """
    if apps.is_installed('modeltranslation'):
        return import_module('modeltranslation.admin').TabbedExternalJqueryTranslationAdmin
    return admin.ModelAdmin


def get_admin_inline_base_class():
    """
    Return base inline admin class
    """
    if apps.is_installed('modeltranslation'):
        class TabbedTranslationInline(
            import_module('modeltranslation.admin').TranslationGenericStackedInline
        ):
            class Media:
                js = (
                    'modeltranslation/js/force_jquery.js',
                    '//ajax.googleapis.com/ajax/libs/jqueryui/1.11.2/jquery-ui.min.js',
                    '//cdn.jsdelivr.net/jquery.mb.browser/0.1/jquery.mb.browser.min.js',
                    'modeltranslation/js/tabbed_translation_fields.js',
                )
                css = {
                    'all': ('modeltranslation/css/tabbed_translation_fields.css',),
                }
        return TabbedTranslationInline
    return GenericStackedInline


def get_html_field_widget():
    """
    Return custom widget for text fields from settings
    """
    default_widget = forms.Textarea()

    if SEO_HTML_ADMIN_WIDGET:
        try:
            custom_widget = getattr(
                import_module(SEO_HTML_ADMIN_WIDGET['widget_path']),
                SEO_HTML_ADMIN_WIDGET['widget']
            )
        except (ImportError, KeyError):
            return default_widget
        else:
            return custom_widget()

    return default_widget



import mimetypes
from typing import Dict

from django.contrib.sites.shortcuts import get_current_site
from django.utils.translation import to_locale, get_language

from ..const import DEFAULT_OBJECT_TYPES, DEFAULT_TWITTER_TYPES
from ..settings import (
    SEO_DEFAULT_IMAGE,
    SEO_IMAGE_HEIGHT,
    SEO_IMAGE_WIDTH,
    SEO_SITE_NAME,
    SEO_FB_APP_ID
)

__all__ = (
    'SeoTagsMixin',
)


class SeoTagsMixin:
    """
    Mixin for seo tags in the <head> section
    """

    SEO_IMAGE_FIELD = 'image'

    def get_robots_content(self) -> str:
        """
        Return robots content
        """
        index = getattr(self, 'index', None)
        follow = getattr(self, 'follow', None)
        if index and follow:
            return f'{index}, {follow}'
        return ''

    def get_canonical(self) -> str:
        """
        Return canonical URL
        """
        return getattr(self, 'canonical', None)

    def get_meta_title(self) -> str:
        """
        Return meta title
        """
        return getattr(self, 'title', '')

    def get_meta_description(self) -> str:
        """
        Return meta description
        """
        return getattr(self, 'description', '')

    def get_meta_keywords(self) -> str:
        """
        Return meta keywords
        """
        return getattr(self, 'keywords', '')

    def get_facebook_app_id(self) -> str:
        """
        Return facebook app id
        """
        return SEO_FB_APP_ID

    def get_meta_image_field(self):
        """
        Return image field instance to get image url
        """
        return getattr(self, self.SEO_IMAGE_FIELD, None)

    def get_meta_image(self) -> str:
        """
        Return url of image
        """
        image_field = self.get_meta_image_field()
        if image_field:
            try:
                return image_field.url
            except Exception:
                return SEO_DEFAULT_IMAGE
        return SEO_DEFAULT_IMAGE

    def get_meta_image_alt(self) -> str:
        """
        Return alternative text for image
        """
        return getattr(self, 'alt', self.get_meta_title())

    def get_image_width(self) -> int:
        """
        Return width value for image
        """
        return getattr(self, 'width', SEO_IMAGE_WIDTH)

    def get_image_height(self) -> int:
        """
        Return height value for image
        """
        return getattr(self, 'height', SEO_IMAGE_HEIGHT)

    def get_opengraph_type(self) -> str:
        """
        Return open graph object type
        """
        return getattr(self, 'object_type', DEFAULT_OBJECT_TYPES[0][0])

    def get_twitter_type(self) -> str:
        """
        Return open graph object type
        """
        if hasattr(self, 'twitter_type'):
            return self.twitter_type
        return getattr(self, 'twitter_type', DEFAULT_TWITTER_TYPES[0][0])

    def get_h1_title(self) -> str:
        """
        Return  h1 title
        """
        return getattr(self, 'h1', '')

    def get_seo_text(self) -> str:
        """
        Return  top text
        """
        return getattr(self, 'seo_text', '')

    def as_meta(self, request) -> Dict[str, str]:
        """
        Return dict available to render meta tags
        """
        meta = {
            'robots': self.get_robots_content(),
            'canonical': (
                self.get_canonical() or
                request.build_absolute_uri()
            ),
            'title': self.get_meta_title(),
            'description': self.get_meta_description(),
            'keywords': self.get_meta_keywords(),

            'facebook_app_id': self.get_facebook_app_id(),
            'og_type': self.get_opengraph_type(),
            'site_name': SEO_SITE_NAME or get_current_site(request),
            'og_locale': to_locale(get_language()),

            'twitter_type': self.get_twitter_type(),
            'request': request
        }
        image = self.get_meta_image()
        if image:
            meta.update({
                'image': request.build_absolute_uri(image),
                'alt': self.get_meta_image_alt(),
                'mime_type': mimetypes.guess_type(image)[0] or 'image/jpeg',
                'image_width': self.get_image_width(),
                'image_height': self.get_image_height(),
            })
        return meta



from django.contrib import admin
from django.utils.translation import gettext_lazy as _

from ..settings import SEO_MODELS

__all__ = (
    'ContentObjectListFilter',
    'SeoModelsFilter',
)


class ContentObjectListFilter(admin.SimpleListFilter):
    title = _('Content object')

    # Parameter for the filter that will be used in the URL query.
    parameter_name = 'content_object_is_none'

    def lookups(self, request, model_admin):
        """
        Returns a list of tuples. The first element in each
        tuple is the coded value for the option that will
        appear in the URL query. The second element is the
        human-readable name for the option that will appear
        in the right sidebar.
        """
        return (
            ('yes', _('Exists')),
            ('no', _("Doesn't exist")),
        )

    def queryset(self, request, queryset):
        """
        Returns the filtered queryset based on the value
        provided in the query string and retrievable via
        `self.value()`.
        """
        if self.value() == 'yes':
            ids = []
            for obj in queryset.iterator():
                if obj.content_object is None:
                    ids.append(obj.pk)
            return queryset.filter(id__in=ids)

        if self.value() == 'no':
            ids = []
            for obj in queryset.iterator():
                if obj.content_object is not None:
                    ids.append(obj.pk)
            return queryset.filter(id__in=ids)


class SeoModelsFilter(admin.SimpleListFilter):
    title = _('Seo model')
    parameter_name = 'seo_model'

    def lookups(self, request, model_admin):
        seo_models = set(SEO_MODELS)
        return [(model, model.title()) for model in seo_models]

    def queryset(self, request, queryset):
        if self.value():
            return queryset.filter(content_type__model=self.value())



import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import seo.mixins.models
import seo.utils


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='ModelInstanceSeo',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('facebook_app_id', models.CharField(blank=True, max_length=40, verbose_name='Facebook app id')),
                ('object_type', models.CharField(blank=True, choices=[('website', 'Website'), ('article', 'Article')], max_length=40, verbose_name='Open graph type')),
                ('twitter_type', models.CharField(blank=True, choices=[('summary', 'Summary Card'), ('summary_large_image', 'Summary Card with Large Image'), ('player', 'Player'), ('app', 'App')], max_length=40, verbose_name='Twitter type')),
                ('index', models.CharField(blank=True, choices=[('index', 'Index'), ('noindex', 'No index')], max_length=15, verbose_name='Index robots value')),
                ('follow', models.CharField(blank=True, choices=[('follow', 'Follow'), ('nofollow', 'No follow')], max_length=15, verbose_name='Follow robots value')),
                ('title', models.CharField(blank=True, max_length=255, verbose_name='Seo Title')),
                ('keywords', models.TextField(blank=True, verbose_name='Meta Keywords')),
                ('description', models.TextField(blank=True, verbose_name='Meta Description')),
                ('image', models.ImageField(blank=True, null=True, upload_to=seo.utils.image_upload_to, validators=[django.core.validators.FileExtensionValidator(['jpg', 'jpeg', 'png'])], verbose_name='Изображение')),
                ('alt', models.CharField(blank=True, max_length=255, verbose_name='Alt')),
                ('object_id', models.CharField(help_text='Please enter the ID of the related object.', max_length=255, verbose_name='Object Primary Key')),
                ('content_type', models.ForeignKey(help_text='Please select the type (model) for the relation, you want to build.', limit_choices_to={'model__in': ['article']}, on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType', verbose_name='Content Type')),
            ],
            options={
                'verbose_name': 'Model instance seo',
                'verbose_name_plural': 'Model instance seo',
            },
            bases=(seo.mixins.models.SeoTagsMixin, models.Model),
        ),
        migrations.CreateModel(
            name='ViewSeo',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('facebook_app_id', models.CharField(blank=True, max_length=40, verbose_name='Facebook app id')),
                ('object_type', models.CharField(blank=True, choices=[('website', 'Website'), ('article', 'Article')], max_length=40, verbose_name='Open graph type')),
                ('twitter_type', models.CharField(blank=True, choices=[('summary', 'Summary Card'), ('summary_large_image', 'Summary Card with Large Image'), ('player', 'Player'), ('app', 'App')], max_length=40, verbose_name='Twitter type')),
                ('index', models.CharField(blank=True, choices=[('index', 'Index'), ('noindex', 'No index')], max_length=15, verbose_name='Index robots value')),
                ('follow', models.CharField(blank=True, choices=[('follow', 'Follow'), ('nofollow', 'No follow')], max_length=15, verbose_name='Follow robots value')),
                ('title', models.CharField(blank=True, max_length=255, verbose_name='Seo Title')),
                ('keywords', models.TextField(blank=True, verbose_name='Meta Keywords')),
                ('description', models.TextField(blank=True, verbose_name='Meta Description')),
                ('image', models.ImageField(blank=True, null=True, upload_to=seo.utils.image_upload_to, validators=[django.core.validators.FileExtensionValidator(['jpg', 'jpeg', 'png'])], verbose_name='Изображение')),
                ('alt', models.CharField(blank=True, max_length=255, verbose_name='Alt')),
                ('view', models.CharField(choices=[('index', 'Index')], max_length=100, unique=True, verbose_name='View')),
            ],
            options={
                'verbose_name': 'View seo',
                'verbose_name_plural': 'View seo',
            },
            bases=(seo.mixins.models.SeoTagsMixin, models.Model),
        ),
        migrations.AlterUniqueTogether(
            name='modelinstanceseo',
            unique_together={('content_type', 'object_id')},
        ),
    ]



from django import forms
from django.db import models

from .utils import get_html_field_widget

__all__ = (
    'AdminRichTextFieldMixin',
)


class AdminRichTextFieldMixin:
    """
    Convert given text fields to rich text fields
    """
    richtext_fields = [
        'seo_text'
    ]

    def get_richtext_fields(self):
        return self.richtext_fields

    def formfield_for_dbfield(self, db_field, **kwargs):
        if self.is_text_field(db_field):
            richtext_fields = self.get_richtext_fields()
            if richtext_fields and db_field.name in set(richtext_fields):
                return db_field.formfield(widget=get_html_field_widget())
            return db_field.formfield(widget=forms.Textarea())
        return super().formfield_for_dbfield(db_field, **kwargs)

    def is_text_field(self, field):
        return type(field) == models.TextField



from django.urls import reverse
from django.utils.timezone import localtime, now
from django.utils.text import slugify

__all__ = (
    'image_upload_to',
    'admin_change_url'
)


def image_upload_to(instance, filename) -> str:
    """
    Util to set upload_to path,
    based on model's class name and date of uploading for image field
    """
    tz_now = localtime(now()).strftime('%Y/%m/%d')
    return f"{slugify(instance.__class__.__name__)}/{tz_now}/{filename}"


def admin_change_url(obj) -> str:
    """
    Return admin change url for given object
    """
    app_label = obj._meta.app_label
    model_name = obj._meta.model.__name__.lower()
    return reverse(f'admin:{app_label}_{model_name}_change', args=(obj.pk,))



from .mixins import AdminRichTextFieldMixin
from .utils import get_admin_inline_base_class
from ..models.instance_based import ModelInstanceSeo

__all__ = (
    'ModelInstanceSeoInline',
)


class ModelInstanceSeoInline(AdminRichTextFieldMixin, get_admin_inline_base_class()):
    classes = ['collapse']
    extra = 1
    model = ModelInstanceSeo
    max_num = 1



from django.utils.translation import pgettext_lazy

__all__ = (
    'DEFAULT_IMAGE_EXTENSIONS',
    'DEFAULT_OBJECT_TYPES',
    'DEFAULT_TWITTER_TYPES',
    'INDEX_CHOICES',
    'FOLLOW_CHOICES'
)

DEFAULT_IMAGE_EXTENSIONS = ['jpg', 'jpeg', 'png']

DEFAULT_OBJECT_TYPES = (
    ('article', pgettext_lazy('OG types', 'Article')),
    ('website', pgettext_lazy('OG types', 'Website')),
)

DEFAULT_TWITTER_TYPES = (
    ('summary', pgettext_lazy('Twitter card types', 'Summary Card')),
    ('summary_large_image', pgettext_lazy('Twitter card types', 'Summary Card with Large Image')),
    ('player', pgettext_lazy('Twitter card types', 'Player')),
    ('app', pgettext_lazy('Twitter card types', 'App')),
)


INDEX_CHOICES = (
    ('index', pgettext_lazy('Index choices', 'Index')),
    ('noindex', pgettext_lazy('Index choices', 'No index')),

)

FOLLOW_CHOICES = (
    ('follow', pgettext_lazy('Follow choices', 'Follow')),
    ('nofollow', pgettext_lazy('Follow choices', 'No follow')),
)



from setuptools import setup, find_packages

pkj_name = 'seo'

with open('requirements.txt') as f:
    requires = f.read().splitlines()


setup(
    name='django-ok-seo',
    version='0.2.5',
    description='Django seo app',
    long_description=open('README.md').read(),
    author='Oleg Kleschunov',
    author_email='igorkleschunov@gmail.com',
    url='https://github.com/LowerDeez/ok-seo',
    packages=[pkj_name] + [pkj_name + '.' + x for x in find_packages(pkj_name)],
    include_package_data=True,
    license='MIT',
    install_requires=requires,
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3'
    ]

)



import django.core.validators
from django.db import migrations, models
import seo.utils


class Migration(migrations.Migration):

    dependencies = [
        ('seo', '0002_auto_20181126_2038'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='modelinstanceseo',
            name='bottom_text',
        ),
        migrations.RemoveField(
            model_name='modelinstanceseo',
            name='top_text',
        ),
        migrations.RemoveField(
            model_name='viewseo',
            name='bottom_text',
        ),
        migrations.RemoveField(
            model_name='viewseo',
            name='top_text',
        ),
        migrations.AddField(
            model_name='modelinstanceseo',
            name='height',
            field=models.PositiveIntegerField(default=630, verbose_name='Image height'),
        ),
        migrations.AddField(
            model_name='modelinstanceseo',
            name='seo_text',
            field=models.TextField(blank=True, help_text='Can be useful for some static pages or some objects (like product category).', verbose_name='Seo text for page'),
        ),
        migrations.AddField(
            model_name='modelinstanceseo',
            name='width',
            field=models.PositiveIntegerField(default=1200, verbose_name='Image width'),
        ),
        migrations.AddField(
            model_name='viewseo',
            name='height',
            field=models.PositiveIntegerField(default=630, verbose_name='Image height'),
        ),
        migrations.AddField(
            model_name='viewseo',
            name='seo_text',
            field=models.TextField(blank=True, help_text='Can be useful for some static pages or some objects (like product category).', verbose_name='Seo text for page'),
        ),
        migrations.AddField(
            model_name='viewseo',
            name='width',
            field=models.PositiveIntegerField(default=1200, verbose_name='Image width'),
        ),
        migrations.AlterField(
            model_name='modelinstanceseo',
            name='follow',
            field=models.CharField(blank=True, choices=[('follow', 'Follow'), ('nofollow', 'No follow')], default='follow', max_length=15, verbose_name='Follow robots value'),
        ),
        migrations.AlterField(
            model_name='modelinstanceseo',
            name='image',
            field=models.ImageField(blank=True, null=True, upload_to=seo.utils.image_upload_to, validators=[django.core.validators.FileExtensionValidator(['jpg', 'jpeg', 'png'])], verbose_name='Image'),
        ),
        migrations.AlterField(
            model_name='modelinstanceseo',
            name='index',
            field=models.CharField(blank=True, choices=[('index', 'Index'), ('noindex', 'No index')], default='index', max_length=15, verbose_name='Index robots value'),
        ),
        migrations.AlterField(
            model_name='modelinstanceseo',
            name='object_type',
            field=models.CharField(blank=True, choices=[('article', 'Article'), ('website', 'Website')], default='article', max_length=40, verbose_name='Open graph type'),
        ),
        migrations.AlterField(
            model_name='modelinstanceseo',
            name='twitter_type',
            field=models.CharField(blank=True, choices=[('summary', 'Summary Card'), ('summary_large_image', 'Summary Card with Large Image'), ('player', 'Player'), ('app', 'App')], default='summary', max_length=40, verbose_name='Twitter type'),
        ),
        migrations.AlterField(
            model_name='viewseo',
            name='follow',
            field=models.CharField(blank=True, choices=[('follow', 'Follow'), ('nofollow', 'No follow')], default='follow', max_length=15, verbose_name='Follow robots value'),
        ),
        migrations.AlterField(
            model_name='viewseo',
            name='image',
            field=models.ImageField(blank=True, null=True, upload_to=seo.utils.image_upload_to, validators=[django.core.validators.FileExtensionValidator(['jpg', 'jpeg', 'png'])], verbose_name='Image'),
        ),
        migrations.AlterField(
            model_name='viewseo',
            name='index',
            field=models.CharField(blank=True, choices=[('index', 'Index'), ('noindex', 'No index')], default='index', max_length=15, verbose_name='Index robots value'),
        ),
        migrations.AlterField(
            model_name='viewseo',
            name='object_type',
            field=models.CharField(blank=True, choices=[('article', 'Article'), ('website', 'Website')], default='article', max_length=40, verbose_name='Open graph type'),
        ),
        migrations.AlterField(
            model_name='viewseo',
            name='twitter_type',
            field=models.CharField(blank=True, choices=[('summary', 'Summary Card'), ('summary_large_image', 'Summary Card with Large Image'), ('player', 'Player'), ('app', 'App')], default='summary', max_length=40, verbose_name='Twitter type'),
        ),
    ]
